

#include <qapplication.h>
#include <qboxlayout.h>

#include <qmainwindow.h>
#include <qgraphicslayout.h>

#include "chart.h"


QLineChart::QLineChart ( QWidget *parent ): QWidget( parent )
{
    QFont qfLabel;

    setParent( parent );
    
    qfLabel.setPointSize( 20 );
    qfLabel.setBold( true );

    qcvView = new QChartView ( &qcChart, this );
//    qcvView->setParent( this );
    qcvView->setRenderHint( QPainter::Antialiasing );
    
    qcChart.addSeries( &qlsSeries );
    qcChart.addAxis( &qvaAxisX, Qt::AlignBottom );
    qcChart.addAxis( &qvaAxisY, Qt::AlignLeft );

//    qlsSeries.append( 5, 1 );
    qlsSeries.setName( qsFL1 ( "Line" ) );
    qlsSeries.attachAxis( &qvaAxisX );
    qlsSeries.attachAxis( &qvaAxisY );
/*    QPen green(Qt::red);
    green.setWidth(3);
    qlsSeries->setPen(green); */

//    qvaAxisX.setLabelsFont( qfLabel );
    qvaAxisX.setTickCount( 5 );
    qvaAxisX.setRange( 0, 10 );
    qvaAxisX.setReverse ( true );
    
//  qvaAxisY.setLabelsFont( qfLabel );
    qvaAxisY.setRange ( -5, 1 );

    qcChart.setAnimationOptions( QChart::NoAnimation );
//    qcChart.setTitle( qsFL1 ("Dynamic line") );
    qcChart.legend()->hide();
    qcChart.setMargins( QMargins( 0, 0, 0, 0 ) );
    qcChart.layout()->setContentsMargins( 0, 0, 0, 0 );
//    qcChart.setVisible( true );


/*    setGeometry( 0, 0, 250, 250 );
    qcvView.setGeometry( 0, 0, 250, 250 ); */
//    qcChart.setGeometry( 0, 0, 250, 250 );


    qlLegend = new QLabel ( qcvView );

    updateLegend();

    
//    show();
        
    charData.init();
}

QLineChart::~QLineChart()
{
}

void QLineChart::setMinMaxValue( QValueAxis *qvaValue,  QPointF *qpfValue, struct T_CharData &charData )
{
    qreal qrAxisMax, qrAxisMin, qrAxisMaxRif, qrAxisMinRif, qrAxisMaxNew, qrAxisMinNew;
    qreal qrRifMaxMax, qrRifMaxMin, qrRifMinMax, qrRifMinMin;
    qreal qrAxisRange, qrMaxStep, qrMinStep, qrAutoStep;

    qrAutoStep = 0.05;
 
    if ( qpfValue )
    {
        if ( qrealCompare( qpfValue->y(), charData.qrMaxValue  ) > 0 )
            charData.qrMaxValue = qpfValue->y();

        if ( qrealCompare( qpfValue->y(), charData.qrMinValue  ) < 0 )
            charData.qrMinValue = qpfValue->y();
    }

    if ( qvaValue )
    {
        qrAxisMaxNew = qrAxisMaxRif = qrAxisMax = qvaValue->max();
        qrAxisMinNew = qrAxisMinRif = qrAxisMin = qvaValue->min();
        qrAxisRange = qrAxisMax - qrAxisMin;


        if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        {
            qrRifMaxMin = charData.qrMaxValue + abs ( floor ( ( charData.qrMaxValue * qrAutoStep ) ) );
            qrRifMaxMin = charData.qrMaxValue + abs ( floor ( ( qrAxisRange * qrAutoStep ) ) );

            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMin ) > 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue *  qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( qrAxisRange *  qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) < 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }
        } // if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        else
        {
            qrRifMaxMax = charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue * qrAutoStep ) );
            qrRifMaxMax = charData.qrMaxValue + abs ( ceil ( qrAxisRange * qrAutoStep ) );

            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMax ) < 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue * qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( qrAxisRange * qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) > 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }

            if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
                qvaValue->setMax( qrAxisMaxNew );
        } // else ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )


        qrAxisMaxNew = qrAxisMaxRif;
        if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
            qvaValue->setMax( qrAxisMaxNew );
        


        if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        {
            qrRifMinMin = charData.qrMinValue - abs ( ceil ( ( charData.qrMinValue * qrAutoStep ) ) );
            qrRifMinMin = charData.qrMinValue - abs ( ceil ( ( qrAxisRange * qrAutoStep ) ) );

            if ( qrealCompare( qrAxisMinRif, qrRifMinMin ) < 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs (floor ( charData.qrMinValue *  qrMinStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs (floor ( qrAxisRange *  qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) > 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }
        } // if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        else
        {
            qrRifMinMax = charData.qrMinValue - abs ( floor ( charData.qrMinValue * qrAutoStep ) );
            qrRifMinMax = charData.qrMinValue - abs ( floor ( qrAxisRange * qrAutoStep ) );

            if ( qrealCompare( qrAxisMinRif, qrRifMinMax ) > 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs ( floor ( charData.qrMinValue * qrMinStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs ( floor ( qrAxisRange * qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) < 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }

            if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
                qvaValue->setMin( qrAxisMinNew );
        } // else ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )


        qrAxisMinNew = qrAxisMinRif;
        if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
            qvaValue->setMin( qrAxisMinNew );



        charData.qrOldMaxValue = charData.qrMaxValue;
        charData.qrOldMinValue = charData.qrMinValue;

    } // if ( qvaValue )
} // void setMinMaxValue( QValueAxis *qvaValue,  QPointF *qpfValue, struct T_CharData &charData )

/***************************************************************************
    add chart value
 ***************************************************************************/

void QLineChart::addValue( qreal qrValue )
{
    bool enabOver, enabAdd, enabFirst;

    QPointF qpfPoint;

    QList<QPointF>::reverse_iterator riQlPoint, riQlSuccPoint;

    charData.qrMinValue = qrealMAX;
    charData.qrMaxValue = qrealMIN;

    enabOver = enabAdd = false;
    enabFirst = true;

    riQlSuccPoint = charData.dataList.rend();

    for ( riQlPoint = charData.dataList.rbegin(); !enabOver || riQlPoint != charData.dataList.rend(); !enabOver ? riQlPoint++ : riQlPoint )
    {
        enabOver = riQlPoint == charData.dataList.rend();

        if ( enabFirst )
        {
            if ( riQlPoint != charData.dataList.rend() )
                qpfPoint = *riQlPoint;

            if ( qrealCompare( qpfPoint.x(), maxAxisX  ) < 0 )
                enabAdd = true;

            enabFirst = false;
        }

        if ( !enabOver )
        {
            qpfPoint = *riQlPoint;

            qpfPoint.setX ( qpfPoint.x() + stepAxisX );

            if ( enabAdd )
            {
                *riQlPoint = qpfPoint;

                setMinMaxValue ( nullptr, &qpfPoint, charData );
            }
            else
            {
                if ( riQlSuccPoint != charData.dataList.rend() )
                {
                    *riQlSuccPoint = qpfPoint;

                    setMinMaxValue ( nullptr, &qpfPoint, charData );
                }

                riQlSuccPoint = riQlPoint;
            }
        }
    }

    if ( enabAdd )
        charData.dataList.prepend( qpfPoint );

    if ( !charData.dataList.isEmpty() )
    {
        qpfPoint.setX ( minAxisX );
        qpfPoint.setY ( qrValue );
        charData.dataList.first() = qpfPoint;

        setMinMaxValue ( nullptr, &qpfPoint, charData );
    }


    setMinMaxValue ( &qvaAxisY, nullptr, charData );
    updateSeries( &qlsSeries );

} // void addValue( qreal qrValue )

/***************************************************************************
    set current value
 ***************************************************************************/

void QLineChart::setValue( QString qsValue )
{
    qlLegend->setText( qsValue );
}

/***************************************************************************
    update legend
 ***************************************************************************/

void QLineChart::updateLegend()
{
    qlLegend->setText( qsFL1 ( "Line" ) );
    qlLegend->move( 0, ( ( size().height() - qlLegend->height() ) / 10.0 ) );
    qlLegend->setFixedWidth( size().width() );
    qlLegend->setAlignment( Qt::AlignHCenter );
    qlLegend->setVisible( true );
}

/***************************************************************************
    update chart series
 ***************************************************************************/

void QLineChart::updateSeries( QAbstractSeries *series )
{    
    QXYSeries *qxySeries;
    
    if ( series )
    {
        qxySeries = static_cast<QXYSeries *>( series );

        qxySeries->replace( charData.dataList );
    }
}

/***************************************************************************
    widget resize event
 ***************************************************************************/

void QLineChart::resizeEvent ( QResizeEvent *qreEvent )
{
    Q_UNUSED(qreEvent);

    qcvView->setGeometry( 0, 0, width(), height() );

    updateLegend();
}




testChart::testChart() : QWidget()
{
    setWindowTitle ( qsFL1 ( "Chart (only for alpha test)" ) );
    
        QPalette palette3, palette4;
        QBrush brush3, brush4;
        
        brush3 = QBrush (QColor(15, 57, 241, 255));
        brush3.setStyle(Qt::SolidPattern);
                
        palette3.setBrush(QPalette::Active, QPalette::Window, brush3);
        palette3.setBrush(QPalette::Inactive, QPalette::Window, brush3);
        palette3.setBrush(QPalette::Disabled, QPalette::Base, brush3);
        palette3.setBrush(QPalette::Disabled, QPalette::Window, brush3);
        
        brush4 = brush3;
        brush4.setColor(QColor(200, 57, 241, 255));

        palette4 = palette3;
        palette4.setBrush(QPalette::Active, QPalette::Window, brush4);
        palette4.setBrush(QPalette::Inactive, QPalette::Window, brush4);
        palette4.setBrush(QPalette::Disabled, QPalette::Base, brush4);
        palette4.setBrush(QPalette::Disabled, QPalette::Window, brush4);

        qwChart = new QWidget(this);
        qwChart->setObjectName("qwChart3");
        qwChart->setGeometry(QRect(0, 0, 250, 250));

        qfChart = new QFrame(qwChart);
        qfChart->setObjectName("qfChart3");
//        qfChart->setGeometry(QRect(0, 0, 250, 250));
        qfChart->resize(qwChart->size());

        qfChart->setPalette(palette3);
        qfChart->setAutoFillBackground(true);
//        qfChart3->setFrameShape(QFrame::Shape::Panel);
//        qfChart3->setFrameShadow(QFrame::Shadow::Raised);

        qwChart2 = new QWidget(this);
        qwChart2->setObjectName("qwChart4");
        qwChart2->setGeometry(QRect(300, 0, 250, 250));

        qfChart2 = new QFrame(qwChart2);
        qfChart2->setObjectName("qfChart4");
//        qfChart2->setGeometry(QRect(0, 0, 250, 250));
        qfChart2->resize(qwChart2->size());

        qfChart2->setPalette(palette4);
        qfChart2->setAutoFillBackground(true);
        qfChart2->setFrameShape(QFrame::Shape::Panel);
        qfChart2->setFrameShadow(QFrame::Shadow::Raised);
                
//        show();
        
//        qclChart = new QLineChart (qfChart);

        qclChart2 = new QLineChart (qfChart2);
        qclChart2->setGeometry( 0, 0, 250, 250 );

    
//    show();        
                
    resize(600, 600);

    posY = 0;
//    qclChart->addData( posY );
    qclChart2->addValue( posY );
    qclChart2->setValue( qsFL1( "" ) );


    idTimer = startTimer( 100 );

//   setCentralWidget(qwChart3);

/*   QMainWindow * win = (QMainWindow *) QApplication::activeWindow();        

     foreach (QWidget *w, qApp->topLevelWidgets())
        if (win = qobject_cast<QMainWindow*>(w))
          break; */
}

testChart::~testChart()
{

}

void testChart::timerEvent( QTimerEvent * )
{
    static int enabKill = 0;

    if ( posY == 0.0 )
        posY = 2.0;

    posY += stepY;

    if ( enabKill >= 0 && enabKill < 10 )
    {
        if ( posY > 3 )
        {        
            posY = -20.0;

            if ( enabKill == 1 )
                enabKill = 9;

            enabKill++;
        }
    }
    else if ( enabKill >= 10 && enabKill < 12 )
    {
        if ( posY > 30.0 )
        {
            posY = 9.0;
            enabKill++;
        }
    }
    else if ( enabKill < 15 )
    {
        if ( posY > 12.0 )
        {
            posY = 9.0;

            enabKill++;
        }
    }

//    qclChart->addData( posY );
    qclChart2->addValue( posY );
    qclChart2->setValue( qsFL1( "TCTL %1 C" ).arg ( posY, 0, 'f', 0, QChar::fromLatin1( ' ' ) ) );
}
