/***************************************************************************                          
    file:           generalcfg.cpp
    description:    general configuration
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qgroupbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcheckbox.h>

#include "lmsensorswidget.h"
#include "ksensorsmain.h"
#include "klocalizedstring.h"

#include "general.h"
#include "generalcfg.h"


GeneralCfg::GeneralCfg( ksensorsMain *kmMainIn, QWidget *parent, const QString &name) : GeneralCfgDesign(parent,name)
{
    kmMain = kmMainIn;

    palPanelCfg= new PaletteCfg( boxGeneralPalette );

 boxGeneralPalette->setAlignment( Qt::Vertical );
// boxGeneralPalette->layout()->setSpacing( 6 );
// boxGeneralPalette->layout()->setMargin( 12 );
/* QGridLayout *boxGeneralPaletteLayout = new QGridLayout( this );
 boxGeneralPaletteLayout->setAlignment( Qt::AlignTop );
 boxGeneralPaletteLayout->addWidget( palPanelCfg, 0, 0 ); */

  readGeneralInfo();

 //connect( buttonApply ,SIGNAL(clicked()),this, SLOT(slotApplyChanges()));
  
    connect( qtwMain, SIGNAL( currentChanged( int ) ), this, SLOT( slotMainTabPageChanged( int ) ));  
}

GeneralCfg::~GeneralCfg(){
}

void GeneralCfg::applyGeneralChanges()
{  
    QString panelSizeText;
  
  palPanelCfg->savePalette();
   
    panelSizeText = qcbPanelSize->currentText();  
    panelSizeText.replace ( QRegularExpression( qsFL1( " x .*" ) ), qsFL1( "" ) );
    if ( panelSizeText.toInt() <= 0 )
        panelSizeText = qsFL1( "64" );
    
    setGroupConf ( qsFL1( "ShowWidget" ) );  
    setEntryConf ( qsFL1( "PanelsSize" ), panelSizeText.toInt() );
    setEntryConf ( qsFL1( "panelSolid" ), qcbPanelSolid->isChecked() );
    setEntryConf ( qsFL1( "panelOpacityValue" ), qdsbPanelOpacityValue->value() );
    setEntryConf ( qsFL1( "windowsSolid" ), qcbWindowsSolid->isChecked() );
    setEntryConf ( qsFL1( "windowsOpacityValue" ), qdsbWindowsOpacityValue->value() );
  
    setGroupConf ( qsFL1( "General" ) );
    setEntryConf ( qsFL1( "AutoStart" ), CheckBoxAutoStart->isChecked() );
}

void GeneralCfg::applyPreferencesChanges()
{    
    setGroupConf ( qsFL1( "General" ) );
    setEntryConf ( qsFL1( "UpdateInterval" ), qdsbUpdTime->value(), 0, ksensorsSetting::DeleteDefault );
}

void GeneralCfg::readGeneralInfo()
{
    int panelSize; 
    QString panelSizeText;

    palPanelCfg->readPalette( qsFL1( "" ), qsFL1( "" ), qsFL1( "" ) );

    setGroupConf ( qsFL1( "ShowWidget" ) );
    panelSize = getEntryConf ( qsFL1( "PanelsSize" ), -1, ksensorsSetting::AutoDefault );

    panelSizeText.clear();;
    panelSizeText.append ( QString::number( panelSize ) ).append ( qsFL1( " x " ) ).append ( QString::number ( panelSize ) );
     
    qcbPanelSize->setCurrentText( tr2i18n ( panelSizeText.toLatin1().data() ) );

    bool panelSolid = getEntryConf ( qsFL1( "panelSolid" ), false, ksensorsSetting::AutoDefault );
    qcbPanelSolid->setChecked( panelSolid );

    double panelOpacityValue = getEntryConf ( qsFL1( "panelOpacityValue" ), -1.0, ksensorsSetting::AutoDefault );
    qdsbPanelOpacityValue->setValue( panelOpacityValue );

    bool windowsSolid = getEntryConf ( qsFL1( "windowsSolid" ), false, ksensorsSetting::AutoDefault );
    qcbWindowsSolid->setChecked( windowsSolid );

    double windowsOpacityValue = getEntryConf ( qsFL1( "windowsOpacityValue" ), -1.0, ksensorsSetting::AutoDefault );
    qdsbWindowsOpacityValue->setValue( windowsOpacityValue );


    setGroupConf ( qsFL1( "General" ) );
    CheckBoxAutoStart->setChecked( getEntryConf ( qsFL1( "AutoStart" ), false, ksensorsSetting::AutoDefault ) );
    qdsbUpdTime->setValue ( getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, true ) );    
}

void GeneralCfg::readPreferencesInfo()
{
    setGroupConf ( qsFL1( "General" ) );
    qdsbUpdTime->setValue ( getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, true ) );
}

void GeneralCfg::slotMainTabPageChanged( int index )
{
    Q_UNUSED( index );
  
    switch ( qtwMain->currentIndex() )
    {
    case 0: 
        readGeneralInfo();  
        
        break;
    case 1: 
        readPreferencesInfo();                       
    
        break;
    }
}

void GeneralCfg::slotApplyChanges()
{
    if (!isVisible())
        return;

    switch ( qtwMain->currentIndex() )
    {
    case 0: 
        applyGeneralChanges();      
        
        break;
    case 1:       
        applyPreferencesChanges(); 
      
      break;
    }
    
    kmMain->updateSensorsPanelsDocks ( qsFL1( "General" ), false );
}

void GeneralCfg::showEvent ( QShowEvent *event )
{
  Q_UNUSED(event);
  
  readGeneralInfo();
}
