/***************************************************************************
    file:           ksensorsmain.h
    description:    ksensors main
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _KSENSORSMAIN_H_
#define _KSENSORSMAIN_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qobject.h>
#include <qpointer.h>
#include <qmenu.h>
#include <qdialog.h>

#include "ksensorssetting.h"

#include "sensordevsgroups.h"
#include "sensorpanel.h"
#include "lmsensorswidget.h"
#include "lmsensordockpanel.h"
#include "lmsensorsalarms.h"

#include "ksensorscfg.h"

class KSensorsCfg;

class LMSensorDockPanel;

/**
  *@original author Miguel Novas
  */

class ksensorsMain : public QObject, public ksensorsSetting
{

	Q_OBJECT

public:
    enum Update 
    {
//          = 0x00000000,
        GlobalTimer = 0x00000001,
        ReadConfig = 0x00000002,
        UpdateIntervalList = 0x00000004,
        UpdateSensorsList = 0x00000008
//          = 0x00000010,
//          = 0x00000012       
    };

    Q_DECLARE_FLAGS(Updates, Update)

    struct T_UpdateData
    {
        bool withChange;
        QString sensorName;
        QList<int> updateIntervalList;

        void init()
        {
            withChange = false;
            sensorName.clear();
            updateIntervalList.clear();
        }
    };

	ksensorsMain(QObject *parent = nullptr, const QString &name = QString() );
	~ksensorsMain();

  void init();
  void createMenu ( QMenu *&qmMenu, QAction *&qaMinRest );

  sensorDevsGroups *getSensorDGList();
  
public Q_SLOTS:
  void saveConfig ( bool enabSync );	

    void updateSensorsPanelsDocks( const QString &name,bool withChange );

protected Q_SLOTS:

  void mouseEventReceived(QMouseEvent *e);
  void minimizeRestoreWidget();
  void createConfigWidget();
  void updateMenu ( bool enabMinimize );
  void showAbout();
  void close();

private:

    int countSensors;

    sensorDevsGroups *sensorDGList;

  QPointer<KSensorsCfg>     sensorsCfg;
  QPointer<LMSensorsWidget> sensorsWidget;
  QPointer<LMSensorsAlarms> sensorsAlarm;

  QMenu *qmContext;
  QAction *qaMinimizeRestore;

  int idTimer;
  int updateInterval;
  int updateIntervalTimer;
  
  void updateSensorsStatus ( Updates updatesType, struct T_UpdateData &updateData );
  void createWidgets();
  void createDockWidgets();  
  void createDockSensor(Sensor *sensor);
  void deleteDockSensor(LMSensorDockPanel *sensor);
  void createShowWidget(int desktop=0);
  void createAlarmControl();

  int maxCommonDivisor ( QList<int> &intList );
  
  void timerEvent( QTimerEvent * ) override;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(ksensorsMain::Updates)

#endif // _KSENSORSMAIN_H
