/***************************************************************************
                          lmsensorswidget.cpp  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qobject.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qevent.h>
#include <qicon.h>
#include <qapplication.h>
#include <qdebug.h>

#include <kaboutdata.h>

#include "general.h"
#include "lmsensorscfg.h"
#include "lmsensorswidget.h"


/***************************************************************************
 *  Public methods                                                         *
 ***************************************************************************/

LMSensorsWidget::LMSensorsWidget(sensorDevsGroups *lsensors,QWidget *parent, const QString &name )
                                :  QWidget( parent, Qt::WindowStaysOnTopHint /* ,name */)

{
  Q_UNUSED(name);
  
/*   WStyle_Customize | WStyle_StaysOnTop | WStyle_NormalBorder | */
//  setWindowFlags( Qt::Widget );
  setAttribute( Qt::WA_DeleteOnClose );  

  sysPanels = nullptr;
  
 panelsSize= 64;
 setMinimumSize(0,0);
/* setSizeIncrement(64,64);
 setBaseSize(64,64); */
 //
 sensors= lsensors;
 //
 childDraging = nullptr;
 //
 const KAboutData kadAboutData = KAboutData::applicationData();
 
 setWindowTitle( kadAboutData.displayName() );
 //
 windowsSolid = true;
 qpmBackGround = QPixmap ( findIcon ("ksensorsbackground.png") );
 qpLMSensorsWidget = palette();
 loadPalette();

    setSizeIncrement( 64, 64 );
    setBaseSize( 64, 64);
 //
 
 setGroupConf ( qsFL1( "ShowWidget" ) ); 
 QRect rect = getEntryConf ( qsFL1( "geometry" ) , QRect(), ksensorsSetting::AutoDefault );

 move( rect.topLeft() );
 resize( rect.size() );
 
 loadGeneralOptions( true );
 
 // 
// setBackgroundPixmap( QPixmap( findIcon ("ksensorsbackground.png") ) ); 
 //
  
 //
 createSensorPanels();
 //
 
// make windowHandle() find struct QTLWExtra -> QWidgetWindow *window not set to zero
  WId wiForSetIncr = winId();
  Q_UNUSED(wiForSetIncr);
/*  qtResize.setSingleShot(true);
  connect(&qtResize, SIGNAL(timeout()), this, SLOT(resizeDone())); */

}

LMSensorsWidget::~LMSensorsWidget()
{  
  saveGeometry();
}

void LMSensorsWidget::updateWidget ( const QString &name, bool withChange )
{
    bool withShow;

    if ( !name.isEmpty() && name != qsFL1( "General" ) )
    {
        Panel *panel= findChild<Panel *>(name);
        Sensor *sensor= sensors->getSensor(name);

        if( sensor )
        {
//          setGroupConf ( name );
            setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );

            withShow = getEntryConf ( qsFL1( "show" ) , false, ksensorsSetting::AutoDefault );

            if ( withShow != (bool)panel )
                withChange = true;

          if ( withChange ) 
          {
              if(panel) 
                  delete panel; 
            
              if ( withShow )
                (void)new SensorPanel( sensor, this, sensor->getLongName() );
          }
      } 

      panel= findChild<Panel *>(name);
    if(panel) loadPalette(panel);
    } 
    else 
    {
    loadPalette();
    loadGeneralOptions( false );
    }
}

// ******************* Protected Q_SLOTS ******************************


/* bool LMSensorsWidget::event(QEvent *event)
{
//    qDebug() << "\033[91m" << "qDebug(): LMSensorsWidget::event event->type()" << event->type() <<"\033[0m";
  
    switch (event->type())
    {
//    case QEvent::NonClientAreaMouseButtonRelease:
    case QEvent::Resize:
//        if (event->spontaneous()) return false;   
    case QEvent::UpdateRequest:
        qtResize.stop();
        qtResize.start(500);

        break;          
    default:
      
      break;
    }
  
  return QWidget::event (event);
}

void LMSensorsWidget::resizeDone()
{
    QSize qsResize, qsSnap, qsObj;
    QRect qrGeom;
  
    qsResize = size();    
    qsSnap.setWidth(int((qsResize.width() / 64.0) + 0.5) * 64.0);
    qsSnap.setHeight(int((qsResize.height() / 64.0) + 0.5) * 64.0);

//    if (qsSnap != qsResize)
    {
      resize(qsResize.width(), qsResize.height() -1);
      resize(qsSnap);
    }
} */


/***************************************************************************
 * Private methods                                                         *
 ***************************************************************************/

void LMSensorsWidget::childEvent(QChildEvent *e)
{
 if(e->added()) {
   loadDisplayConfig((Panel *)e->child());
 }
}

// ************* Draging panels and show menu ************************

void LMSensorsWidget::childEventReceived(QEvent *e)
{
QMouseEvent *m = (QMouseEvent*)e;

 switch(e->type()) {
  case QEvent::MouseButtonPress:

      if( m->button()==Qt::LeftButton ) startDragChild(m,(QWidget *)sender());
      break;

  case QEvent::MouseButtonRelease:

      if( m->button()==Qt::RightButton ) Q_EMIT rightMouseClicked(m);
      break;
  default:
    
    break;
 }
}


void LMSensorsWidget::startDragChild(QMouseEvent *m, QWidget *w)
{
	if(childDraging == nullptr )
	{
		childDraging= w;    
		grabMouse(Qt::SizeAllCursor);
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
		origMousePt = w->mapFromGlobal(m->globalPos());
#else
                origMousePt = w->mapFromGlobal(m->globalPosition().toPoint());
#endif
		//w->move( m->globalPos()-mapToGlobal(QPoint(0,0))-origMousePt);
		w->raise();
		w->update();
	}
}

void LMSensorsWidget::endDragChild()
{
int x= childDraging->x();
int y= childDraging->y();

 if(x<0) x=0;
 if(y<0) y=0;
 releaseMouse();
 childDraging->move(
                     ((x+childDraging->width() /2) / panelsSize) * panelsSize ,
                     ((y+childDraging->height()/2) / panelsSize) * panelsSize
                   );
 saveDisplayConfig((Panel *)childDraging);
 childDraging= nullptr;
}

void LMSensorsWidget::mouseMoveEvent ( QMouseEvent *m )
{
	if(childDraging) 
	{ 
		childDraging->move(m->pos()-origMousePt);
		childDraging->update(); 
	}
}


void LMSensorsWidget::mouseReleaseEvent ( QMouseEvent *m )
{
 if(childDraging && m->button()==Qt::LeftButton)  endDragChild();
 else
   if(m->button()==Qt::RightButton) Q_EMIT rightMouseClicked(m);
}

// ********************** Panels creation ************************

void LMSensorsWidget::findUnusedPosition(int *px,int *py)
{
bool found;
uint mx,my,x,y,a[32];
  const QObjectList *list= &children();

 for(y=0; y<32; y++) a[y]= 0;
 mx= 0; my= 0;
 if(list)
 {
    for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it )       
    {
      QWidget *obj = (QWidget *)(*it);      
       
       if ( obj )
       {

       y=obj->y()/panelsSize;
       x=obj->x()/panelsSize;
       if(x<32 && y<32) a[y]|=  (1 << x);
       if(x>mx) mx= x;
       if(y>my) my= y;
       
       }
       else
         break;
      
    }
 
 }
 //
 for(x=0,y=0,found=false;y<=my && !found;) {
   found= ((a[y] & (1 << x))==0);
   if(!found)
     if(++x>mx) { x= 0; y++; }
 }

 if(!found) {
    x= (width()-1) / panelsSize;
    if(x>mx) { x= mx+1; y= 0;    }
    else     { x= 0;    y= my+1; }
 }
 //
 *px= x;
 *py= y;
}

void LMSensorsWidget::loadDisplayConfig(Panel *display)
{
 if( display == nullptr ) return;

 loadPalette(display);

 bool fNoCfg;
 QPoint p;
 
    Sensor *sensor= sensors->getSensor( display->objectName() );
//  setGroupConf ( display->objectName() );
    setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
    p = getEntryConf ( qsFL1( "showPos" ) , QPoint(), ksensorsSetting::AutoDefault );
 
 fNoCfg= (p.x()<0 || p.y()<0);
 if(fNoCfg) findUnusedPosition(&p.rx(),&p.ry());
 display->move(p.x()*panelsSize,p.y()*panelsSize);
 display->resize(panelsSize,panelsSize);
 display->show();
 if(fNoCfg) {
    saveDisplayConfig(display);
    adjustSize();
 }
 display->installEventFilter(this);
 connect(display, SIGNAL(eventReceived(QEvent *)), this, SLOT(childEventReceived(QEvent *)) );
}

void LMSensorsWidget::saveDisplayConfig(Panel *display)
{
 setGroupConf ( display->objectName() );
 setEntryConf ( qsFL1( "show" ) ,true);
 QPoint p= display->pos()/panelsSize;
 setEntryConf ( qsFL1( "showPos" ) ,p);
}


void LMSensorsWidget::createSensorPanels()
{
 const QObjectList *chips= sensors->getSensorsChips();
 if(chips)
 {
    for ( QObjectList::ConstIterator it = chips->begin(); it != chips->end(); ++it )       
    {
      SensorsList *chip = (SensorsList *)(*it);
       
      if ( chip )
      {
        if ( chip->getClass() == Sensor::sysSensor )
          sysPanels = (SYSSensorsList *)chip;
        
       QObjectList *list= (QObjectList *)chip->getSensors();
       if(list) 
       {
          for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it )       
          {
            LMSensor *sensor = (LMSensor *)(*it);
       
            if ( chip )
            {
//              setGroupConf ( sensor->objectName() );
                setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
             if ( getEntryConf ( qsFL1( "show" ) , false, ksensorsSetting::AutoDefault ) )
                (void) new SensorPanel(sensor, this, sensor->getLongName() );
              
            }
            else
              break;
            
           }
       }
      }
      else
        break;
      
    }
    
 }
}

void LMSensorsWidget::updatePanels()
{
    bool enabSize;
    int newSize;
  
    setGroupConf ( qsFL1( "ShowWidget" )  );
    newSize = getEntryConf ( qsFL1( "PanelsSize" ) , -1, ksensorsSetting::AutoDefault );

  const QObjectList *panels= &children();

  enabSize = false;
  
  if (panelsSize != newSize) 
  {   
    setSizeIncrement(newSize,newSize);
    setBaseSize(newSize,newSize);
    enabSize = true;
  }
    
  if ( panels )
  {
    for ( QObjectList::ConstIterator it = panels->begin(); it != panels->end(); ++it )       
    {
      Panel *panel = (Panel *)(*it);
      QWidget *qwPanel = (QWidget *)(*it);
       
      if ( qwPanel )
      {

        if ( enabSize )
        {
          qwPanel->move( (qwPanel->x()/panelsSize)*newSize, (qwPanel->y()/panelsSize)*newSize);
          qwPanel->resize(newSize,newSize);
        }
      
        loadPalette ( panel ) ;
      }
      else
        break;
      
    }      
  } // if ( panels )
   
  if ( enabSize )
  {
    panelsSize= newSize;
    adjustSize();
  }
} // void updatePanels()


void LMSensorsWidget::saveGeometry()
{
  QRect geometry ( pos(), size() );
  
  if ( !(geometry.x() >= 0 && geometry.y() >= 0) )
  {
    geometry.setX( 0 );
    geometry.setY( 0 );
  }

  setGroupConf ( qsFL1( "ShowWidget" )  );
  setEntryConf ( qsFL1( "geometry" ) , geometry );

//  void LMSensorsWidget::cfgWriteGeometry( QRect geometry )

}

void LMSensorsWidget::loadGeneralOptions( bool enabForce )
{
  bool newWindowsSolid;
  double windowsOpacityValue;
  
  setGroupConf ( qsFL1( "ShowWidget" )  );
  newWindowsSolid = getEntryConf ( qsFL1( "windowsSolid" ) , false, ksensorsSetting::AutoDefault );
  windowsOpacityValue = getEntryConf ( qsFL1( "windowsOpacityValue" ) , -1.0, ksensorsSetting::AutoDefault );

  if ( newWindowsSolid != windowsSolid  || enabForce )
  {
    windowsSolid = newWindowsSolid;

    setAttribute( Qt::WA_TranslucentBackground, !windowsSolid );
    setAttribute( Qt::WA_NoSystemBackground, !windowsSolid );
    setWindowFlags( ( windowsSolid ? ( Qt::WindowStaysOnTopHint | Qt::WindowTitleHint ) : ( Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint | Qt::Tool ) ) );

//    setVisible( true );
    show();
  }
  setWindowOpacity( ( windowsOpacityValue / 100.0 ) );


  updatePanels();
}

// ************************+******** Palettes stuff ****************************

void LMSensorsWidget::loadPalette(Panel *panel)
{
  QPalette pal = qpLMSensorsWidget;
  if(panel) 
  {
    if(getPalette(pal, panel->objectName() ,false)) 
      panel->setPalette(pal);
    else  
      panel->setPalette( qpLMSensorsWidget /* QPalette() */ );
    
    panel->updateFrame();
  } 
  else 
  {
    getPalette(pal, QString(), true);
    qpLMSensorsWidget = pal;
    setPalette(pal);
    pal.setBrush( backgroundRole(), QBrush( qpmBackGround ));    
    setPalette(pal);
  }
}

/* void LMSensorsWidget::setBackgroundPixmap( const QPixmap &qpmBackGround )
{  
  QPalette pal = palette();
  pal.setBrush( backgroundRole(), QBrush( qpmBackGround ));
  setPalette(pal);
} */
