/***************************************************************************
                          sensorslist.cpp  -  description
                             -------------------
    begin                : mi� abr 24 2002
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcoreevent.h>
#include <qdebug.h>

#include "general.h"
#include "lmsensorswidget.h"
#include "sensorslist.h"


SensorsList::SensorsList(QObject *parent, const QString &name)
                                  : QObject(/* parent */ /* ,name */)
{
   setObjectName ( name );
   
 tempScale= Sensor::dgCelsius;

 monitorized = false;
 
    idTimer = 0;
    
    confUpdateInterval = 0;
    sensorUpdateInterval = 5000;    
    timerUpdateInterval = sensorUpdateInterval;
    withGlobalTimerUpdate = false;
 
 setParent ( parent ); 
}

SensorsList::~SensorsList()
{
}


void SensorsList::setMonitorized(bool enable)
{
 if(monitorized!=enable) {
   monitorized= enable;
   if(enable)
   {
/*      if ( !idTimer )
         idTimer = startTimer( updateInterval ); */
   }
   else
   {
      if ( idTimer )
      {
         killTimer ( idTimer );
         idTimer = 0;
      }
   }
 }
}

bool SensorsList::getMonitorized()
{
    return monitorized;
}

void SensorsList::setTempScale(Sensor::TempScale scale)
{
 if(tempScale!=scale) {
     tempScale= scale;
 }
}

/***************************************************************************
    set and get of the configuration and sensor update interval
 ***************************************************************************/

void SensorsList::setConfUpdateInterval( double seconds )
{
    confUpdateInterval = ( seconds * 1000 );
}

double SensorsList::getConfUpdateInterval ( bool enabSec )
{
    return ( confUpdateInterval / ( enabSec ? 1000.0 : 1.0 ) );
}

void SensorsList::setSensorUpdateInterval( double seconds )
{
    int updateIntervalNew;

    updateIntervalNew = ( seconds * 1000 );

    if( sensorUpdateInterval != updateIntervalNew )
    {
        sensorUpdateInterval = updateIntervalNew;
        timerUpdateInterval = sensorUpdateInterval;
    
    if(monitorized) {
       setMonitorized(false);
       setMonitorized(true);
    }
    }
}

double SensorsList::getSensorUpdateInterval( bool enabSec )
{
    return ( sensorUpdateInterval / ( enabSec ? 1000.0 : 1.0 ) );
}

void SensorsList::readConfig()
{
    double minSensorUpdateInterval, minConfUpdateInterval;
    QString str;

//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( getShortName(), getLongName(), getSubName() );
    if ( sensorClass == Sensor::SensorClass::hdSensor )
    {
        minSensorUpdateInterval = getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, true );
        minConfUpdateInterval = getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), 5.0, false );
        if ( doubleCompare( minConfUpdateInterval,  minSensorUpdateInterval ) < 0 )
            minConfUpdateInterval = 0.0;
        else
            minSensorUpdateInterval = minConfUpdateInterval;

        setConfUpdateInterval ( minConfUpdateInterval );
        setSensorUpdateInterval ( minSensorUpdateInterval );
    }
    else
    {
        setConfUpdateInterval ( getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, false ) );
        setSensorUpdateInterval ( getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, true ) );
    }

    str = getEntryConf ( qsFL1( "Scale" ), QString(), ksensorsSetting::AutoDefault );
 
 Sensor::TempScale tempScale;
 if(str== qsFL1( "F" ) ) tempScale= Sensor::dgFahrenheit;
 else if(str== qsFL1( "K" ) ) tempScale= Sensor::dgKelvin;
      else             tempScale= Sensor::dgCelsius;
 setTempScale(tempScale);
}

void SensorsList::writeConfig()
{
//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( getShortName(), getLongName(), getSubName() );
    setEntryConf ( qsFL1( "ShortName" ),  getShortName() );
    setEntryConf ( qsFL1( "SubName" ),  getSubName() );    
    setEntryConf ( qsFL1( "UpdateInterval" ), ( confUpdateInterval / 1000.0 ), 0, ksensorsSetting::DeleteDefault );

    if ( getClass() != Sensor::sysSensor )
    {
   
 QString str;
 switch(tempScale) {
  case Sensor::dgCelsius   : str = QChar::fromLatin1( 'C' ); break;
  case Sensor::dgKelvin    : str = QChar::fromLatin1( 'K' ); break;
  case Sensor::dgFahrenheit: str = QChar::fromLatin1( 'F' ); break;
  default:
 
   break;
 }
      setEntryConf ( qsFL1( "Scale" ), str);
 
    }
}

/***************************************************************************
    get/set class, names e description, object name of sensor device
 ***************************************************************************/

void SensorsList::setClass( Sensor::SensorClass newClass )
{
    sensorClass = newClass;
}

Sensor::SensorClass SensorsList::getClass()
{
    return sensorClass;
}

void SensorsList::setShortName( const QString &name )
{
    shortName = name;
}

QString SensorsList::getShortName()
{
    return shortName;
}
    
void SensorsList::setLongName( const QString &name )
{
    longName = name;
}

QString SensorsList::getLongName()
{
    return longName;
}

void SensorsList::setSubName( const QString &name )
{
    subName = name;
}

QString SensorsList::getSubName()
{
    return subName;
}

void SensorsList::setDescription( const QString &desc )
{
    description = desc;
}

const QString &SensorsList::getDescription()
{
    return description;
}

void SensorsList::setObjectName( const QString &name )
{
    QObject::setObjectName( name );
    
    setShortName( name );
    setLongName( name );
}

/***************************************************************************
    global timer update
 ***************************************************************************/

void SensorsList::globalTimerUpdate ( int updateIntervalIn, int updateIntervalTimerIn )
{
    int oldTimerInterval;

    Q_UNUSED ( updateIntervalIn );
    
//    qDebug() << "\033[91m" << "qDebug(): SensorsList::globalTimerUpdate()" << objectName() << updateIntervalIn << updateIntervalTimerIn <<  "\033[0m";

    if ( !withGlobalTimerUpdate && timerUpdateInterval > 0 )
    {
//        qDebug() << "\033[91m" << "qDebug(): SensorsList::globalTimerUpdate()" << "start" << timerUpdateInterval << "\033[0m";

        withGlobalTimerUpdate = true;

        oldTimerInterval = timerUpdateInterval;

        timerUpdateInterval -= updateIntervalTimerIn;

        if ( oldTimerInterval > 0 && timerUpdateInterval <= 0 )
        {
//            qDebug() << "\033[91m" << "qDebug(): SensorsList::globalTimerUpdate()" << "updateSensors start" << withGlobalTimerUpdate <<  "\033[0m";

            updateSensors ( false );

            timerUpdateInterval = sensorUpdateInterval;

//            qDebug() << "\033[91m" << "qDebug(): SensorsList::globalTimerUpdate()" << "updateSensors end" << withGlobalTimerUpdate << "\033[0m";
        }
        else
            setWithGlobalTimerUpdate( false );
      
//        qDebug() << "\033[91m" << "qDebug(): SensorsList::globalTimerUpdate()" << "end" << timerUpdateInterval << "\033[0m";

    } // if ( !withGlobalTimerUpdate && timerUpdateInterval > 0 )
} // void globalTimerUpdate ( int updateIntervalIn, int updateIntervalTimerIn )

/***************************************************************************
    set global timer update active state
 ***************************************************************************/

void SensorsList::setWithGlobalTimerUpdate ( bool withGlobalTimerUpdateIn )
{
    withGlobalTimerUpdate = withGlobalTimerUpdateIn;
}

//****************************************************************************
// Protected methods
//****************************************************************************

void SensorsList::timerEvent( QTimerEvent * )
{
//   qDebug() << "\033[91m" << "qDebug(): SensorsList::timerEvent" << idTimer << "\033[0m";
   
    updateSensors ( false );
 
//   qDebug() << "\033[91m" << "qDebug(): SensorsList::timerEvent" << idTimer << "post" << "\033[0m";
}

//***************

void SensorsList::childEvent ( QChildEvent *e )
{   
//   qDebug() << "\033[91m" << "qDebug(): SensorsList::childEvent()" << "\033[0m";
   
 if(e->added()) {
    
//      qDebug() << "\033[91m" << "qDebug(): SensorsList::childEvent()" << "e->added()" <<"\033[0m";

    connect((Sensor *)e->child(),SIGNAL(valueChanged()) ,this,SLOT(slotValueChanged()));
 }
}

//****************************************************************************
// Protected Q_SLOTS
//****************************************************************************

void SensorsList::slotValueChanged()
{
    Q_EMIT valueChanged( (Sensor *)sender() );
}


/*********************************************************************************/
